#include <stdint.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include "lcd.h"
#include "lcdb.h"
#include "rtcasync.h"

#define SPASS

#define POINTCHAR 'o' /* 0x6f */
#define BLANKCHAR ' ' /* 0x20 */

#define COL1 0
#define COL2 4
#define COL3 8
#define COL4 12
#define COLDOT 7
#define COLSEC 15

#define KEYPORT PORTB
#define KEYDDR  DDRB
#define KEYPIN  PINB
#define KEYMINUS PINB0
#define KEYPLUS  PINB1

#define KEYCNTMAX  0xf0
/* <5*10ms key-pressed is bounce */
#define KEYCNTMIN  5
/* 100*10ms key-pressed is "long" */
#define KEYCNTLONG 100
/* check every 50*10 ms for key-press to update time */
#define DTCHECKKEY 50

volatile uint8_t gHeartBeat10ms = 0;
volatile uint8_t gKeyPlusCnt = 0, gKeyMinusCnt = 0;

#ifdef SPASS

void mydelay(unsigned char v)
{
	unsigned char j;
	unsigned int i;
	for (j=0;j<v;j++) for (i=0;i<2000;i++) asm volatile ("nop"::);
}
#endif

//SIGNAL(SIG_OUTPUT_COMPARE1A)
ISR(TIMER1_COMPA_vect)
{
	//se verifica apasarea tastei -
	if ( !(KEYPIN & (1<<KEYMINUS)) ) {
		if (gKeyMinusCnt < KEYCNTMAX) gKeyMinusCnt++;
	}
	else gKeyMinusCnt = 0;
	
	//se verifica apasarea tastei+
	if ( !(KEYPIN & (1<<KEYPLUS)) ) {
		if (gKeyPlusCnt < KEYCNTMAX) gKeyPlusCnt++;
	}
	else gKeyPlusCnt = 0;
	
	gHeartBeat10ms++;
}

void init_keys(void)
{
	uint8_t sreg;

	KEYDDR &= ~( (1<<KEYMINUS) | (1<<KEYPLUS) ); // setezi pinii ca input*/
	KEYPORT |= ( (1<<KEYMINUS) | (1<<KEYPLUS) ); 
	
	sreg=SREG;
	cli();
	
	/* init Timer1 (ISR called ca. every 10ms @ 1 MHz Sys-Clock ) */
	OCR1A  = 4;     /* set top */
	/* CTC Mode 4 Prescaler 1024 */
	TCCR1B = (1<<WGM12)|(1<<CS12)|(1<<CS10); 
	TIMSK |= (1<<OCIE1A); /* enable output-compare int */
	TCNT1 = 0; /* reset counter */
	
	SREG=sreg;
}

void check_keys(void)
{
	uint8_t sreg;
	uint8_t updaterequired;
	static uint8_t tlastchange = 0;
	
	
	if ( ( gKeyPlusCnt < KEYCNTLONG ) && ( gKeyMinusCnt < KEYCNTLONG ) ) {
		if ( (uint8_t)(gHeartBeat10ms - tlastchange) < DTCHECKKEY ) return;
	}
	
	sreg=SREG;
	
	updaterequired=0;
	
	/* se incrementeaza timpul */
	if ( gKeyPlusCnt > KEYCNTMIN ) {
		cli();	
		updaterequired=1; 
		RTC_m++;
		if (RTC_m==60) {
			RTC_m=0; 
			RTC_h++;
			if (RTC_h>23) RTC_h=0;
		}
	}
	
	/* se decrementeaza timpul */
	if ( gKeyMinusCnt > KEYCNTMIN ) {
		cli();
		updaterequired=1;
		if (RTC_m==0) {
			RTC_m=59;
			if (RTC_h==0) RTC_h=23;
			else RTC_h--;
		}
		else RTC_m--;
	}
	
	if (updaterequired) {
		RTC_s = 0; /* reset seconds */
		RTC_secondchanged = 1; /* flag to update display */
		RTC_minutechanged = 1; /* flag to update display */
		tlastchange=gHeartBeat10ms; /* save change time */
	}

	SREG=sreg;
}

void clockpoints(uint8_t sec)
{
	uint8_t c;
	lcd_gotoxy(COLDOT,1);
	c = ( !(sec & 0x01) ) ? POINTCHAR : BLANKCHAR;
	lcd_putc(c);
	lcd_gotoxy(COLDOT,2);
	
	lcd_putc(c);
}

void showseconds(uint8_t sec)
{
	lcd_gotoxy(COLSEC,1);
	lcd_putc(sec/10+'0');
	lcd_gotoxy(COLSEC,2);
	lcd_putc(sec%10+'0');
}

int main(void)
{
#ifdef SPASS
	uint8_t i;
#endif
			
	
	MCUCSR|=_BV(JTD);
	MCUCSR|=_BV(JTD);

	lcd_init(LCD_DISP_ON); /* initialize display, cursor off */
	LCDBigInit(); /* upload CGRAM (user defined characters) */

	init_keys(); /* init input-keys */

	
	RTC_init(); /* init async RTC with 32kHz-Crystal */

#ifdef SPASS
	
	
	lcd_clrscr();
	clockpoints(0);
	i=10;
	do {
		i--;
		LCDBigNumWrite(i,COL1); /* params: number, column */
		LCDBigNumWrite(i,COL2); 
		LCDBigNumWrite(i,COL3); 
		LCDBigNumWrite(i,COL4); 
		mydelay(30);
	} while (i>0);
	lcd_clrscr();
	lcd_gotoxy(3,1);
	lcd_puts_P("Go! Go! GO!");
	mydelay(100);
#endif

	sei(); /* enable interrupts */
	
	lcd_clrscr();

	RTC_set(11,54,59); /* start-time, first LCD-update at 11:55:00 */
	
	/* main loop */
	for (;;)                            /* loop forever */
	{	
		if (RTC_secondchanged) {
			RTC_secondchanged=0;
			clockpoints(RTC_s);
			showseconds(RTC_s);
		}
			
		if (RTC_minutechanged) {
			RTC_minutechanged=0;
			LCDBigNumWrite(RTC_h/10,COL1); /* params: number, column */
			LCDBigNumWrite(RTC_h%10,COL2);
			LCDBigNumWrite(RTC_m/10,COL3);
			LCDBigNumWrite(RTC_m%10,COL4);
		}
		
		check_keys();
	}
	
	return 0; /* never reached */
}


